@@ -7,7 +7,13 @@ const config = {
 	// Consult https://svelte.dev/docs/kit/integrations
 	// for more information about preprocessors
 	preprocess: [vitePreprocess(), mdsvex()],
+
 	kit: {
+		paths: {
+			base: '/lcpp',
+			relative: true
+		},
+		router: { type: 'hash' },
 		adapter: adapter({
 			pages: '../public',
 			assets: '../public',
@@ -17,8 +23,12 @@ const config = {
 		}),
 		output: {
 			bundleStrategy: 'inline'
+		},
+		alias: {
+			$styles: 'src/styles'
 		}
 	},
+
 	extensions: ['.svelte', '.svx']
 };
 
